import requests
import json
from urllib import parse


class DragonAPI(object):
    def __init__(self,
                 host,
                 username='admin',
                 password='dragonadmin',
                 timeout=15,
                 jwt=None):
        self.base_url = "http://{}".format(host)
        self.username = username
        self.password = password
        self.timeout = timeout
        self.jwt = jwt
        if not self.jwt:
            self.auth()

    def post(self, path, data=None):
        response = requests.post(
            parse.urljoin(self.base_url, path),
            headers={'Authorization': 'Bearer ' + self.jwt},
            timeout=self.timeout,
            data=data)
        response.raise_for_status()
        return response.json()

    def get_stream(self, path):
        response = requests.get(
            parse.urljoin(self.base_url, path),
            headers={'Authorization': 'Bearer ' + self.jwt},
            stream=True,
            timeout=self.timeout)
        response.raise_for_status()
        return response

    def auth(self):
        response = requests.post(
            parse.urljoin(self.base_url, '/api/auth'),
            timeout=self.timeout,
            data={'username': 'admin', 'password': 'dragonadmin'})
        response.raise_for_status()
        json = response.json()
        self.jwt = json['jwt']
        return json

    def summary(self):
        return self.post('/api/summary')

    def overview(self):
        return self.post('/api/overview')

    def pools(self):
        return self.post('/api/pools')

    def updatePools(self,
                    pool1,
                    username1,
                    password1,
                    pool2=None,
                    username2=None,
                    password2=None,
                    pool3=None,
                    username3=None,
                    password3=None):
        return self.post('/api/pools',
                         data={
                             'Pool1': pool1,
                             'UserName1': username1,
                             'Password1': password1,
                             'Pool2': pool2,
                             'UserName2': username2,
                             'Password2': password2,
                             'Pool3': pool3,
                             'UserName3': username3,
                             'Password3': password3,
                         })

    def updatePassword(self,
                       user,
                       currentPassword,
                       newPassword):
        return self.post('/api/updatePassword',
                         data={
                             'user': user,
                             'currentPassword': currentPassword,
                             'newPassword': newPassword
                         })

    def network(self):
        return self.post('/api/network')

    def updateNetwork(self,
                      dhcp='dhcp',
                      ipaddress=None,
                      netmask=None,
                      gateway=None,
                      dns=None):
        return self.post('/api/updateNetwork',
                         data={
                             'dhcp': dhcp,
                             'ipaddress': ipaddress,
                             'netmask': netmask,
                             'gateway': gateway,
                             'dns': json.dumps(dns)
                         })

    def type(self):
        return self.post('/api/type')

    def reboot(self):
        return self.post('/api/reboot')

    def poweroff(self):
        return self.post('/api/poweroff')

    def restartCgMiner(self):
        return self.post('/api/restartCgMiner')

    def factoryReset(self):
        return self.post('/api/factoryReset')

    def getAutoTune(self):
        return self.post('/api/getAutoTune')

    def getAutoTuneStatus(self):
        return self.post('/api/getAutoTuneStatus')

    def setAutoTune(self, autotune):
        return self.post('/api/setAutoTune',
                         data={'autotune': autotune})

    def upgradeDownload(self, url):
        return self.post('/upgrade/download',
                         data={'url': url})

    def getLatestFirmwareVersion(self):
        return self.post('/api/getLatestFirmwareVersion')

    def getDebugStats(self):
        return self.post('/api/getDebugStats')

    def streamLogs(self):
        return self.get_stream('/stream/logs')



import vcr
from pytest import fixture
from dragon_rest.dragons import DragonAPI

vcr = vcr.VCR(
    cassette_library_dir='tests/fixtures/cassettes'
)


@fixture
def jwt():
    return 'eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJEcmFnb25NaW50IiwiaWF0IjoxNTI4NjQzOTYwLCJleHAiOjE1Mjg2NjU1NjAsInVzZXIiOiJhZG1pbiJ9.rm9Ll81v5pVmlHQCEyFqoeb6TnJSqH66lM1-izK_NoQ'


@fixture
def host():
    return '10.1.0.25'


@vcr.use_cassette()
def test_auth(host):
    api = DragonAPI(host)

    assert api.jwt is not None


@vcr.use_cassette()
def test_summary(host, jwt):
    api = DragonAPI(host, jwt=jwt)
    r = api.summary()

    assert r['success']
    assert len(r['DEVS']) == 3


@vcr.use_cassette()
def test_overview(host, jwt):
    api = DragonAPI(host, jwt=jwt)
    r = api.overview()

    assert r['success']
    assert r['type'] == 'T1'


@vcr.use_cassette()
def test_pools(host, jwt):
    api = DragonAPI(host, jwt=jwt)
    r = api.pools()

    assert r['success']
    assert len(r['pools']) == 2


@vcr.use_cassette()
def test_updatePools(host, jwt):
    api = DragonAPI(host, jwt=jwt)
    r = api.updatePools(
        pool1='stratum+tcp://us-east.stratum.slushpool.com:3333',
        username1='brndnmtthws.dragon-0ade5',
        password1='x',
        pool2='stratum+tcp://pool.ckpool.org:3333',
        username2='3GWdXx9dfLPvSe7d8UnxjnDnSAJodTTbrt.dragon-0ade5',
        password2='x',
        pool3=None,
        username3=None,
        password3=None
    )

    assert r['success']


@vcr.use_cassette()
def test_updatePassword(host, jwt):
    api = DragonAPI(host, jwt=jwt)
    r = api.updatePassword('admin', 'dragonadmin', 'loladmin')
    assert r['success']
    r = api.updatePassword('admin', 'loladmin', 'dragonadmin')

    assert r['success']


@vcr.use_cassette()
def test_network(host, jwt):
    api = DragonAPI(host, jwt=jwt)
    r = api.network()

    assert r['success']
    assert r['dhcp'] == 'dhcp'


@vcr.use_cassette()
def test_updateNetwork(host, jwt):
    api = DragonAPI(host, jwt=jwt)
    r = api.updateNetwork(dhcp='dhcp')

    assert r['success']


@vcr.use_cassette()
def test_type(host, jwt):
    api = DragonAPI(host, jwt=jwt)
    r = api.type()

    assert r['success']
    assert r['type'] == 'T1'


@vcr.use_cassette()
def test_getAutoTune(host, jwt):
    api = DragonAPI(host, jwt=jwt)
    r = api.getAutoTune()

    assert r['success']
    assert r['autoTuneMode'] == 'efficient'


@vcr.use_cassette()
def test_getAutoTuneStatus(host, jwt):
    api = DragonAPI(host, jwt=jwt)
    r = api.getAutoTuneStatus()

    assert r['success']
    assert r['mode'] == 'efficient'


@vcr.use_cassette()
def test_getAutoTune(host, jwt):
    api = DragonAPI(host, jwt=jwt)
    r = api.setAutoTune('efficient')

    assert r['success']


@vcr.use_cassette()
def test_getLatestFirmwareVersion(host, jwt):
    api = DragonAPI(host, jwt=jwt)
    r = api.getLatestFirmwareVersion()

    assert r['success']


@vcr.use_cassette()
def test_getDebugStats(host, jwt):
    api = DragonAPI(host, jwt=jwt)
    r = api.getDebugStats()

    assert r['success']


# This works, but VCR doesn't work with streams
# @vcr.use_cassette()
# def test_streamLogs(host, jwt):
#     api = DragonAPI(host, jwt=jwt)
#     r = api.streamLogs()
#     for line in r.iter_lines():
#         print(line)
